<?php

/**
 * @file
 * Contains the exceptions class of the TransMission library.
 */

namespace JPResult\TransMission;

/**
 * Provides exceptions specific to TransMission.
 *
 * This is used to throw exceptions specific to TransMission's SOAP service.
 * Possible errors are:
 * - Incorrect login details \n
 *   Indicates that the provided login information in incorrect or incomplete.
 * - Failed to save new shipping job \n
 *   Indicates an unknown error on the remote end.
 * - Shipping job does not exist \n
 *   Indicates that no shipping job with the given unique code exists.
 * - Shipping job cannot be removed \n
 *   Indicates that the shipping job has already been processed, so it can no
 *   longer be removed.
 * - Requested labels do not exist \n
 *   Indicates that no shipping jobs exist with the given unique codes.
 * - The notification could not be saved \n
 *   Indicates a problem validating the request, possibly due to incorrect login
 *   details.
 * - No address found corresponding to this postal code \n
 *   Indicates that the postal code given can not be found in the database.
 * - Incorrect postal code, must consist of 4 digits and 2 letters \n
 *   Indicates that the given postal code is not formatted correctly.
 * - No shipping jobs found \n
 *   Indicates that no shipping jobs match the given criteria.
 * - No status updates found \n
 *   Indicates that no status updates match the given criteria.
 *
 * @todo Flesh out class documentation.
 */
class TransMissionFault extends \Exception {
  /**
   * Parse TransMission's error and create a TransMissionFault object.
   *
   * @param SoapFault $previous
   *   The SoapFault exception generated by the SoapClient.
   */
  function __construct(\SoapFault $previous) {
    // Use the same error code as the SoapFault.
    $code = (int) $previous->faultcode;

    switch ($previous->getMessage()) {
      case 'Onjuiste inloggegevens':
        $message = 'Incorrect login details';
        break;

      case 'Opslaan nieuwe opdracht niet gelukt':
        $message = 'Failed to save new shipping job';
        break;

      case 'Opdracht niet gevonden':
        $message = 'Shipping job does not exist';
        break;

      case 'Opdracht kan niet worden verwijderd':
        $message = 'Shipping job cannot be removed';
        break;

      case 'Opgevraagde labels niet gevonden':
        $message = 'Requested labels do not exist';
        break;

      case 'De vooraanmelding kon niet worden opgeslagen':
        $message = 'The notification could not be saved';
        break;

      case 'Geen opdrachten gevonden':
        $message = 'No shipping jobs found';
        break;

      case 'Onjuiste postcode (Moet bestaan uit 4 cijfers, 2 letters)':
        $message = 'Incorrect postal code, must consist of 4 digits and 2 letters';
        break;

      case 'Geen adres gevonden op deze postcode':
        $message = 'No address found corresponding to this postal code';
        break;

      case 'Geen statussen gevonden':
        $message = 'No status updates found';
        break;

      default:
        // @todo Do more parsing here.
        $message = 'An unknown error occurred';
    }

    parent::__construct($message, $code, $previous);
  }
}
